using System;
using System.Collections.Generic;
using System.Text;
using Utils.Logs;
using System.ComponentModel;
using Utils.DataBase;
using System.Data;
using MySql.Data.MySqlClient;

namespace PhoenixDbAdmin.Class
{
    public class ReportManager
    {
        #region Variables
        /// <summary>
        /// Report ID
        /// </summary>
        private int vID;
        /// <summary>
        /// Creation date of report
        /// </summary>
        private DateTime vDate;
        /// <summary>
        /// Type of the Log (Report)
        /// </summary>
        private string vType = LogType.Report.ToString();
        /// <summary>
        /// Title of the report
        /// </summary>
        private string vTitle;
        /// <summary>
        /// Message of the report
        /// </summary>
        private string vMessage;
        /// <summary>
        /// ID of the user who wrote the report
        /// </summary>
        private int vIDUser;
        /// <summary>
        /// Id the report read?
        /// </summary>
        private bool vIsRead = false;
        /// <summary>
        /// Sql manager
        /// </summary>
        private MySQLDbSql SqlManager;
        #endregion

        #region Constructor
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="_ID">Login for the database</param>
        /// <param name="_Password">Password for the database</param>
        /// <param name="_Server">Server wherin is the database</param>
        /// <param name="_Database">Name of the database</param>
        public ReportManager(string _ID, string _Password, string _Server, string _Database)
        {
            SqlManager = new MySQLDbSql(_ID, _Password, _Server, _Database);
        }
        /// <summary>
        /// Constructor
        /// </summary>
        /// <param name="_ConnectionString">Connection string</param>
        public ReportManager(string _ConnectionString)
        {
            SqlManager = new MySQLDbSql(_ConnectionString);
        }
        #endregion

        #region Methods
        /// <summary>
        /// Select all report in the database
        /// </summary>
        /// <returns>DataSet with all the reports</returns>
        /// <exception cref="MySqlException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public DataSet SelectReport()
        {
            try
            {
                MySQLDbSql SqlManager = new MySQLDbSql("PhoenixUserManag", "", "jibedoubleve.mine.nu", "Phoenix");
                SqlManager.SqlQuery = "SELECT l.*, u.Name, u.Surname FROM Log l, User u WHERE l.IDUser = u.ID;";
                DataSet ds = new DataSet();
                SqlManager.ExecuteSql(ref ds);
                return ds;
            }
            catch (MySqlException ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
            catch (InvalidOperationException ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
        }
        /// <summary>
        /// Select all read or unread report in the database
        /// </summary>
        /// <param name="_IsRead">Indicates whether the read report have to be selected or not</param>
        /// <returns>DataSet with the selected reports</returns>
        /// <exception cref="MySqlException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public DataSet SelectReport(bool _IsRead)
        {
            try
            {
                MySQLDbSql SqlManager = new MySQLDbSql("PhoenixUserManag", "", "jibedoubleve.mine.nu", "Phoenix");
                SqlManager.SqlQuery = "SELECT l.*, u.Name, u.Surname FROM Log l, User u WHERE l.IDUser = u.ID AND";
                if (_IsRead)
                    SqlManager.SqlQuery += " IsRead = 1;";
                else
                    SqlManager.SqlQuery += " IsRead = 0";
                DataSet ds = new DataSet();
                SqlManager.ExecuteSql(ref ds);
                return ds;
            }
            catch (MySqlException ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
            catch (InvalidOperationException ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }

        }
        /// <summary>
        /// Gets the user name on his ID
        /// </summary>
        /// <param name="_ID">ID of the user</param>
        /// <returns>Name and Surname of the user</returns>
        public string GetUserName(int _ID)
        {
            SqlManager.SqlQuery = "SELECT * FROM User WHERE ID = " + _ID + ";";
            DataSet ds = new DataSet();
            SqlManager.ExecuteSql(ref ds);
            return (string)ds.Tables[0].Rows[0]["Name"] + " " + (string)ds.Tables[0].Rows[0]["Surname"];
        }
        /// <summary>
        /// Update the current report with new value to indicates if the report is read or not
        /// </summary>
        /// <param name="_IsRead">TRUE indicates the report is read, FALSE indicates it is unread</param>
        /// <exception cref="MySqlException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        public void SetRead(bool _IsRead)
        {
            try
            {
                ulong val = 0;
                if (_IsRead)
                    val = 1;
                this.SqlManager.SqlQuery = "UPDATE Log SET IsRead = " + val + " WHERE ID = " + this.ID + ";";
                int i = this.SqlManager.ExecuteNonQuery();
            }
            catch (MySqlException ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
            catch (InvalidOperationException ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
        }
        /// <summary>
        /// Gets all the users of written in the database
        /// </summary>
        /// <returns>DataSet with the users</returns>
        /// <exception cref="MySqlException">A connection-level error occurred while opening the connection.</exception>
        /// <exception cref="InvalidOperationException">Cannot execute a command within a transaction context 
        /// that differs from the context in which the connection was originally enlisted.</exception>
        /// <exception cref="ArgumentException">If no user are set in the ReportManager</exception>
        public DataSet GetUsers()
        {
            try
            {
                DataSet ds = new DataSet();
                this.SqlManager.SqlQuery = "SELECT * FROM User;";
                this.SqlManager.ExecuteSql(ref ds);
                return ds;
            }
            catch (MySqlException ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
            catch (InvalidOperationException ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
            catch (ArgumentException ex)
            {
                Log.Instance.Write(ex, this, LogType.Error);
                throw;
            }
        }
        #endregion

        #region Presonal events
        /// <summary>
        /// Occures when data are updated in the database
        /// </summary>
        public event EventHandler Updated;
        /// <summary>
        /// Notification of an update in the database
        /// </summary>
        protected virtual void OnUpdate()
        {
            if (Updated != null)
                Updated(this, new EventArgs());
        }
        #endregion

        #region Properties
        /// <summary>
        /// Gets the name of the report's writer
        /// </summary>
        [DisplayName("1.1- Writer's name")]
        [Category("1- Report")]
        [Description("Indicates the name of the writer of the report")]
        public string UserName
        {
            get
            {
                if (ID < 0) return "";
                return this.GetUserName(this.IDUser);
            }
        }
        /// <summary>
        /// Gets or sets the Date of the report
        /// </summary>
        [Category("1- Report")]
        [Description("Indicates the creation date of the report")]
        [DisplayName("1.2- Creation date")]
        [ReadOnly(true)]
        public DateTime Date
        {
            get { return vDate; }
            set { vDate = value; }
        }
        /// <summary>
        /// Gets or sets the type of the report
        /// </summary>
        [Category("1- Report")]
        [Description("Indicates the type of the report")]
        [DisplayName("1.3- Type")]
        [ReadOnly(true)]
        public string Type
        {
            get { return vType; }
            set { vType = value; }
        }
        /// <summary>
        /// Gets or sets the title of the report
        /// </summary>
        [Category("1- Report")]
        [Description("Indicates the title of the report")]
        [DisplayName("1.4- Title")]
        [ReadOnly(true)]
        public string Title
        {
            get { return vTitle; }
            set { vTitle = value; }
        }
        /// <summary>
        /// Gets or sets the message of the report
        /// </summary>
        [Category("1- Report")]
        [Description("Indicates the message of the report")]
        [DisplayName("1.5 Message")]
        [Browsable(false)]
        public string Message
        {
            get { return vMessage; }
            set { vMessage = value; }
        }
        /// <summary>
        /// Indicates whether the report is read or not (Gets and sets)
        /// </summary>
        [Category("2- Management")]
        [Description("Indicates whether the report is read or not")]
        [DisplayName("2.1- Is read?")]
        public bool IsRead
        {
            get { return vIsRead; }
            set
            {
                try
                {
                    if (value == vIsRead) return;
                    this.SetRead(value);
                    vIsRead = value;
                    OnUpdate();
                }
                catch (Exception) { return; }
            }
        }
        /// <summary>
        /// Gets or sets the ID the the report's writer
        /// </summary>
        [Browsable(false)]
        public int IDUser
        {
            get { return vIDUser; }
            set { vIDUser = value; }
        }
        /// <summary>
        /// Gets or sets the ID of the report
        /// </summary>
        [Browsable(false)]
        public int ID
        {
            get { return vID; }
            set { vID = value; }
        }
        /// <summary>
        /// Gets the connection string of the database
        /// </summary>
        [Browsable(false)]
        public string ConnectionString { get { return this.SqlManager.ConnectionString; } }
        #endregion
    }
}
