using System;
using System.Collections.Generic;
using System.Text;

namespace PhoenixPrescription.Class
{
    /// <summary>
    /// Shapes the text for a MSWord displaying
    /// </summary>
    public class TextShaper
    {
        #region Variables
        private static string vRTFStart = @"{\rtf1\fbidis\ansi\ansicpg1252\deff0\deflang2060{\fonttbl{\f0\froman\fprq2\fcharset0 Times New Roman;}}\viewkind4\uc1\pard\ltrpar ";
        private static string vRTFEnd = "}";
        private static string vFirstRow = @"\f0\fs24";
        private static string vStartRow = @"\ul\b\i";
        private static string vEndRow = @"\ulnone\b0\i0";
        private static string vNextRow = @" \par\par";
        private List<Prescription> vPrescription = new List<Prescription>();
        #endregion

        #region Construtctor
        /// <summary>
        /// Constructor
        /// </summary>
        public TextShaper() { }
        #endregion

        #region Method
        /// <summary>
        /// Process the data to make an RTF Code
        /// </summary>
        /// <returns>RTF binary information</returns>
        private string Process()
        {
            string vBuffer = vRTFStart;
            bool IsTrue = true;
            foreach (Prescription Row in vPrescription)
            {
                if (IsTrue)
                {
                    vBuffer += vStartRow + vFirstRow + " " + Row.Drug + "\\par" + vEndRow + Row.Remark + vNextRow;
                    IsTrue = false;
                }
                else
                    vBuffer += vStartRow + " " + Row.Drug + "\\par" + vEndRow + Row.Remark + vNextRow;
            }
            vBuffer += vEndRow;
            return vBuffer + vRTFEnd;
        }
        #endregion

        #region Properties
        /// <summary>
        /// Gets the field of the prescription word document
        /// </summary>
        public static object[] Fields { get { return new object[] { "Date", "Header", "Patient", "Prescription" }; } }
        /// <summary>
        /// List of the Prescriptions
        /// </summary>
        public List<Prescription> Prescriptions { get { return vPrescription; } set { vPrescription = value; } }
        public string RTF { get { return this.Process(); } }
        #endregion
    }

    /// <summary>
    /// Contains data of a prescription
    /// </summary>
    public struct Prescription
    {
        /// <summary>
        /// Drugs name
        /// </summary>
        private string vDrug;
        /// <summary>
        /// Remarks about a prescription
        /// </summary>
        private string vRemark;
        /// <summary>
        /// Gets or sets drugs name
        /// </summary>
        public string Drug { get { return vDrug; } set { vDrug = value + ":"; } }
        /// <summary>
        /// Gets or sets remarks about a prescription
        /// </summary>
        public string Remark { get { return vRemark; } set { vRemark = value; } }
    }
}
